#INVOICE V5 API
##1 Preface
Package "invoice_v5" purpose is PDF invoice creating from softswitch class4 v5 databases.

It conforms following [requerements doc](https://docs.google.com/presentation/d/1hOQbVMIHvfRo4tCPOkAep73_HHZpQV7p_jZkxWrCE9E/edit#slide=id.p)

Package written by Valentin Novikov for denovolab.
It has http json api, supported OpenAPI v2 REST api specifications.
Api has endpoints for create invoice, get pdf file of created invoice, get json data of invoice (i.e.  when render pdf with another software), get list of invoices etc.
Settings for invoices (both system and client's) will take from database and  out of this project scope. 
"invoice_v5" package contains parts for automate shceduled invoice creation. 
Also invoice_v5 now has  sheduling and sending an alerts functions.
Purpose of implementing is unification both termination and origination invoices and merge conflicing old packages finctions:

 * dnl_invoice_generator 
 * did_billing 
 * invoice_v5(old version) 
 * dnl_ad 


into one module.


##2. Installation
###2.1. Repository of package.

ssh://git@stash.denovolab.com:7999/clas4-v5-rel/invoice-generation.git
http://stash.denovolab.com/scm/clas4-v5-rel/invoice-generation.git
###2.2. Clone package to temporary folder and setup from it.

```shell
myhost ~# mkdir tmp
myhost ~# cd tmp
myhost ~# git clone ssh://git@stash.denovolab.com:7999/clas4-v5-rel/invoice-generation.git
myhost ~# cd invoice-generation/
```

###2.3. Additional postgresql tables may be needed:

`# cat schema/schema.sql | psql -U postgres -d softswitch4v5`

###2.4. conflictiong services cleanup:
admin sholud stop and exclude from distr following services:

[
'dnl_invoice_AutoInvoice',
'dnl_invoice_REST_API',
'dnl_invoice_server',
'dnl_did_invoice_server',
'dnl_ad',
]
```shell
sudo service dnl_invoice_server.service stop
sudo service dnl_invoice_autoinvoice.service stop
sudo service dnl_alerts stop
sudo service dnl_did_billing_invoice.service stop
sudo service dnl_did_billing_server stop
```



###2.4. Sql migration script from "invoice_5" version 1 to version 2 is 
   
`schema/v2.sql`

`# cat schema/v2.sql | psql -U postgres -d softswitch4v5`

It will change "invoice_summary" and "invoice_history" table, 
create "invoice_task" "invoice_debug" tables and cleanup 19 of unused old tables.

###2.5. Setup api

`myhost ~/tmp/invoice-generation # python3 setup.py install`

It should be created a special user for running as api daemon called "webbackend".
Package data root directory by default is

`ls /opt/denovo/inv5/`.

The "webbackend" user should have read-write data root permissions.
Here should be `files` directory for datas and will stored `api.ini` file
Package main scripts are
```shell
ls /usr/local/bin/inv5_*
/usr/local/bin/inv5_run
/usr/local/bin/inv5_worker
/usr/local/bin/inv5_beat
```
###2.6. Running in console single mode
```shell
sudo su webbackend #change user for api permissions
cd /opt/denovo/inv5
/usr/local/bin/inv5_run
```
###2.7. Systemd startup scripts can be found in `etc/systemd` package directory

Its should be copied to root `/etc/systemd/system` and integrated to system:
``` 
# systemctl start dnl_inv5_run.service
# systemctl start dnl_inv5_worker.service
# systemctl start dnl_inv5_beat.service
```

Usually, should be running three deamons inv5_run,inv5_worker,inv5_beat
```shell
ps -A f u | grep inv5
webback+ 20222  0.0  0.1 215336 20724 ?        Ss   Mar12   0:06 /usr/local/bin/python3 /usr/local/bin/inv5_run
webback+ 16073  0.7  0.4 440168 74244 ?        Sl   06:04   2:45  \_ /usr/local/bin/python3 /usr/local/bin/inv5_run
webback+ 20317  0.0  0.5 414128 92048 ?        Ss   Mar12   0:02 /usr/local/bin/python3 /usr/local/bin/inv5_beat
webback+ 16416  0.1  0.5 412740 94756 ?        Ss   06:05   0:27 /usr/local/bin/python3 /usr/local/bin/inv5_worker inv5
webback+ 16442  0.0  0.5 413980 88872 ?        S    06:05   0:00  \_ /usr/local/bin/python3 /usr/local/bin/inv5_worker inv5
webback+ 16443  0.0  0.5 409352 84828 ?        S    06:05   0:00  \_ /usr/local/bin/python3 /usr/local/bin/inv5_worker inv5
webback+ 16444  0.0  0.5 409352 84772 ?        S    06:05   0:00  \_ /usr/local/bin/python3 /usr/local/bin/inv5_worker inv5
webback+ 16446  0.0  0.5 413844 88640 ?        S    06:05   0:00  \_ /usr/local/bin/python3 /usr/local/bin/inv5_worker inv5
```

###2.8. Checking. inv5 api should run on 127.0.0.1:18881.

```shell
# 
netstat -anp |grep 18881
tcp        0      0 127.0.0.1:18881         0.0.0.0:*               LISTEN      16073/python3 
```  
###2.8. Logging 
Normal starting log can be got by `journalctl`:
```shell
journalctl -f -u dnl_inv5_run.service
мар 13 06:03:33 ns389055 inv5_run[20222]: DB [postgresql://webbackend:password@127.0.0.1:5432/softswitch4v5]: Initializing DB, label default
мар 13 06:03:34 ns389055 inv5_run[20222]: [2021-03-13 06:03:34 +0000] [15407] [INFO] Booting worker with pid: 15407
мар 13 06:03:35 ns389055 inv5_run[20222]: 2021-03-13 06:03:35,545 [INFO] 'DB [postgresql://webbackend:password@127.0.0.1:5432/softswitch4v5]: Initializing DB, label default' at model.py: 101
мар 13 06:03:35 ns389055 inv5_run[20222]: 2021-03-13 06:03:35,568 [INFO] 'DB [default]: creating session' at db_class.py: 77

```
For workers log 
```shell
journalctl -f -u dnl_inv5_worker.service
```

If need to enable log to a separate file use setting in `api.ini` file:
```text
[DEFAULT]
log_file = <some log file name>
```

#2 API
##1. Swagger OpenAPI file 
```  
 # curl http://127.0.0.1:18881/inv5/v1/swagger.json

```

##2 Invoice admin functions:
```
POST/invoice_task
GET/invoice_task/{id}
GET/invoice_task/list
GET/client/{client_id}/auto_invoice_test
```
##3 Invoice user functions
```
GET/invoice/list
GET/invoice/{invoice_number}.pdf
GET/invoice/{invoice_number}.json
```

##4. Create manual invoice. POST/invoice_task

This is asynchonus task to create invoice.


mandatory fieds are  
```
client_id	integer
bill_period_start date YYYY-MM-DD
bill_period_end	date YYYY-MM-DD
```
usage

curl -X POST "http://localhost:18881/inv5/v1/invoice_task"   -H "accept: application/json" -H "Content-Type: application/json"   -d "{ \"client_id\": 288, \"bill_period_start\": \"2021-03-01\", \"bill_period_end\": \"2021-03-13\"}"
```json
{
  "success": true,
  "object_id": 44
}
```
returned object_id (here:44) is a task ID running on background. 

those fields are for including additional parts to an invoice:
```
show_acc_summary	boolean
show_payment_applied	boolean
show_daily_summary	boolean
show_inbound_summary	boolean
show_non_rec_charge	boolean
show_country_summary	boolean
show_prefix_summary	boolean
show_rec_charge	boolean
show_trunk_detail	boolean
show_usage_charge	boolean
show_code_name_summary	boolean
show_jd_trunk_detail	boolean
show_jd_daily_summary	boolean
show_top_100tn_summary	boolean
show_did_charge_breakdown	boolean
show_did_charge_summary	boolean
include_scc_charges	boolean
include_tax	boolean
payment_dur_date date YYYY-MM-DD
invoice_zone	string
```
last two fields for configure email action:
```
enable_email	boolean
enable_email_with_cdr	boolean
```
##5. Get task results. GET/invoice_task/{id}

{id} - path parameter, ID of task, returned on previus step.
Usage
```
curl -X GET "http://localhost:18881/inv5/v1/invoice_task/51" -H  "accept: application/json"
{
  "success": true,
  "payload": {
    "status": "Successful",
    "progress": "Summary Successful,File successful",
    "invoice_number": 8903,
    "invoice_num": "93ac0c50dd620dc7b88e5fe05c70e15b",
    "client_id": 288
    "bill_period_start": "2021-03-01",
    "bill_period_end": "2021-03-13",
    "is_auto": false,
    "proc_start_time": "2021-03-13T14:46:27.094228+00:00",
    "proc_end_time": "2021-03-13T14:46:35.046409+00:00",
    "created_on": "2021-03-13T14:46:49.436850+00:00",
  }
}
```
Here is response fields:
```
status - one of Initial, In Process, Successful, Failed
progress - textual task progress message or error result
invoice_number - number of invoice generated by task
invoice_num - same invoice number encoded with md5 hash
client_id - task client
bill_period_start,bill_period_start - invoice period from input parameters
is_auto - True for auto invoice tasks or False for manual ones
proc_start_time,proc_start_time - time when task runned
created_on - creation time of task
```
So UI scenario should watch while status Successful or Failed.

##6. List of task results.   GET/invoice_task/list
Usage   
```shell
curl -X GET "http://localhost:18881/inv5/v1/invoice_task/list" -H  "accept: application/json"
```
```json
{
  "payload": {
    "items": [
      {
        "client_id": 121,
        "created_on": "2021-03-11T21:12:34.485502+00:00",
        "proc_start_time": "2021-03-11T21:12:00.505726+00:00",
        "proc_end_time": "2021-03-11T21:12:13.088669+00:00",
        "invoice_number": 8875,
        "bill_period_end": "2021-03-11",
        "progress": "100",
        "bill_period_start": "2021-03-11",
        "is_auto": true,
        "invoice_num": "959776b99b006e5785c3a3364949ce47",
        "status": "Successful"
      },
      {
        "client_id": 658,
        "created_on": "2021-03-10T04:46:44.298467+00:00",
        "proc_start_time": "2021-03-11T10:05:06.702672+00:00",
        "proc_end_time": "2021-03-11T10:06:33.515358+00:00",
        "invoice_number": null,
        "bill_period_end": "2021-02-28",
        "progress": "Summary Successful,File successful",
        "bill_period_start": "2021-02-01",
        "is_auto": false,
        "invoice_num": null,
        "status": "Successful"
      },
      
      ],
    "total": 54,
    "page": 0,
    "per_page": 30
  },
  "success": true
}
```

Input parameters can be used for filtering this list.

```text
Name	Description
fields string - fields selected only for results
page integer - start page number for UI pagination
per_page integer - page size of records for pagination
order_by string - list ordering field 
order_dir string - sorting direction (ask,desc) default ask
bill_period_end string - bill_period_end filter
bill_period_start string - bill_period_start filter
client_id integer - client_id filter 
client_name string - client name
invoice_number integer - invoice_number filter
is_auto boolean 	
progress string - progress filter 
status string - status filter , one of Initial, In Process, Successful, Failed
created_on_gt datetime - created_on > filter
created_on_lt datetime - created_on < filter 
proc_end_time_gt datetime - proc_end_time > filter
proc_end_time_lt datetime - proc_end_time < filter

```

String field filter can contain glob expressions '*' i.e:

```shell
curl -X GET "http://localhost:18881/inv5/v1/invoice_task/list?client_name=ASHU_*" -H  "accept: application/json"
```
will return all items with client name starting from letters `ASHU_`

##7. Test client auto invoice settigs. GET/invoice/list 
Checks how it will generated in fiture date. GET/client/{client_id}/auto_invoice_test
Usage (test of client_id=57 future date='2021-03-21 00:00:00')
curl -X GET "http://localhost:18881/inv5/v1/client/57/auto_invoice_test?date=2021-03-21%2000%3A00%3A00" -H  "accept: application/json"
```json
{
  "will_generate": false,
  "generated_period": [],
  "messages": "Client v5_egress id 57 SKIP period 2021-03-02 2021-03-31 does not match payment term, or overlapped.last auto invoice was at 2021-03-02 00:00:00",
  "invoice_cycle": "Twice in a month (15th and day)",
  "invoice_cycle_days": [],
  "last_invoice_date": "2021-03-02",
  "invoice_next_date": "2021-03-15",
  "invoice_next_date_of_test": "2021-04-01"
}
```
same test for another date with success generation (future date='2021-03-15 00:00:00')):
curl -X GET "http://localhost:18881/inv5/v1/client/57/auto_invoice_test?date=2021-03-15%2000%3A00%3A00" -H  "accept: application/json"
```json
{
  "will_generate": true,
  "generated_period": [
    {
      "start_time": "2021-03-02T00:00:00",
      "end_time": "2021-03-14T23:59:59"
    }
  ],
  "messages": "Client v5_egress id 57  period 2021-03-02 2021-03-14 success",
  "invoice_cycle": "Twice in a month (15th and last day)",
  "invoice_cycle_days": [],
  "last_invoice_date": "2021-03-02",
  "invoice_next_date": "2021-03-15",
  "invoice_next_date_of_test": "2021-03-15"
}
```

Path parameters:
client_id integer - client to test
Query parameters:
date datetime(YYYY-MM-DD HH:mm:ss) - check if invoice will fired at this date
Output:
```text
will_generate boolean - whether invoice will generated at queried date
generated_period json struct  - period of future invoice (start_time,end_time)
messages string - comments os generation process
invoice_cycle - client's auto invoice scheduling cycle
invoice_cycle_days - client's auto invoice scheduling cycle days
last_invoice_date - client's last auto invoice generated period end
invoice_next_date - default next auto invoice date (current settings)
invoice_next_date_of_test - future next auto invoice date (current settings)
```

##8 Get list of invoices. 
usage
```shell
curl -X GET "http://localhost:18881/inv5/v1/invoice/list?per_page=1&order_by=invoice_number&order_dir=desc" -H  "accept: application/json"
```
```json
{
  "success": true,
  "payload": {
    "items": [
      {
        "invoice_num": "93ac0c50dd620dc7b88e5fe05c70e15b",
        "amount": 490.94,
        "paid": false,
        "client_id": 288,
        "client_name": "sip_ingress",
        "invoice_id": 1185,
        "company_type": "termination",
        "payment_term_name": "daily",
        "invoice_period": "2021-03-01 2021-03-13",
        "due_date": null,
        "client_offset": 0,
        "invoice_end": "2021-03-13T23:59:59+00:00",
        "last_invoice_for": "2021-03-14",
        "data_url": "http://localhost:18881/inv5/v1/invoice/93ac0c50dd620dc7b88e5fe05c70e15b.json",
        "invoice_number": "8903",
        "invoice_time": "2021-03-13T14:47:03+00:00",
        "credit_note": 0,
        "create_type": "manual",
        "state": "normal",
        "invoice_payment": 0,
        "status": "done",
        "url": "http://localhost:18881/inv5/v1/invoice/93ac0c50dd620dc7b88e5fe05c70e15b.pdf",
        "usage_detail_fields": null,
        "client_credit": 0,
        "invoice_start": "2021-03-01T00:00:00+00:00"
      }
    ],
    "total": 671,
    "page": 0,
    "per_page": 1
  }
}

```
Input parameters can be used for filtering this list.
```text
fields string - fields for response
page integer - page number
per_page integer - page size
order_by string field to sort	
    Available values : paid, last_invoice_for, payment_term_name, invoice_end, state, amount, company_name, invoice_number, invoice_period, invoice_start, due_date, invoice_time, invoice_id, create_type, status, client_id, client_name, company_type
order_dir string - Available values : asc, desc
client_id integer - ID of client
client_name string - Name of client
company_type string - Available values : termination, origination, both termination and origination
create_type string - Available values : auto, manual
invoice_id - integer - internal ID of invoice
invoice_number string - number of invoice 
invoice_period string - invoice period 'YYYY-MM-DD - YYYY-MM-DD' 
paid boolean - true if invoice paid
payment_term_name string - name of payment_term schedule
state string - Available values : void, normal, verify, send
status string - Available values : creating, zero cdr, done, only support buy/sell
amount_gt number - amount > 
due_date_gt  string - due_date > 
due_date_lt string - due_date <
invoice_end_gt datetime - invoice_end >
invoice_end_lt datetime - invoice_end <
invoice_id_in - string - comma separated list of invoice ids for selection
invoice_start datetime - invoice_start >
invoice_start_lt datetime - invoice_start <
invoice_time_gt  datetime - (generation time) invoice_time  < 
invoice_time_lt datetime - (generation time) invoice_time  <
last_invoice_for_gt string - (last generated invoice date for this client) last_invoice_for >
last_invoice_for_lt string - (last generated invoice date for this client) last_invoice_for_lt 
```
Respose output fields
```text
{
success	boolean default: true
total	integer  Total items found
page	integer  Current page (starting from 0)
per_page	integer Items per page
items - list of invoices
invoice_id	integer($int32) - unique id of invoice
invoice_number	string - user side number of of invoice
invoice_num	string - invoice_number md5 hash, can be used for secure invoice_number 
create_type	string - Enum:[ auto, manual ] - creation type of invoice 
invoice_time	string($date-time) - invoice generation time
status	string - Enum:[ creating, zero cdr, done, only support buy/sell ]
state	string - Enum:[ void, normal, verify, send ]
url	string - url for get invoice pdf
data_url	string - url for get invoice json data
client_id	integer($int32) - ID of client invoice for
client_name	string - name of client invoice for
company_name	string - name of client's company 
company_type	string - Enum: [ termination, origination, both termination and origination ]
payment_term_name	string - name of client's payment_term schedule
usage_detail_fields	string - fields list of invoice cdr file
invoice_start	string($date-time) - start date of invoice
invoice_end	string($date-time) - end date of invoice
invoice_period	string - invoice period (invoice_start-invoice_end) 'YYYY-MM-DD - YYYY-MM-DD'
due_date	string - payment due date
last_invoice_for	string - last generated invoice date for this client
amount	number - invoice amount
invoice_credit_amount	number($float) 
credit_note	number($float)
client_offset	number($float) 
invoice_payment	number($float) 
client_credit	number($float) 
unpaid_amount	number($float)
paid	boolean  - if invoice is fully paid
```


#8. Get invoice file. GET/invoice/{invoice_number}.pdf

{invoice_number} - can be invoice_number or md5 hashed invoice_num
Usage
```shell

curl -X GET "http://localhost:18881/inv5/v1/invoice/93ac0c50dd620dc7b88e5fe05c70e15b.pdf" -H  "accept: application/pdf"                     
```
respose file content:
```text
%PDF-1.4
%���� ReportLab Generated PDF document http://www.reportlab.com
1 0 obj
<<
/F1 2 0 R /F2 5 0 R
>>
endobj
2 0 obj
<<
/BaseFont /Helvetica /Encoding /WinAnsiEncoding /Name /F1 /Subtype /Type1 /Type /Font
>>
endobj
3 0 obj
```
#8. Get invoice data. GET/invoice/{invoice_number}.json

Usage
```shell
curl -X GET "http://localhost:18881/inv5/v1/invoice/93ac0c50dd620dc7b88e5fe05c70e15b.json" -H  "accept: application/json"
```
Response json file
```json
{
  "acc_summary": {
    "previous_balance": 0,
    "payments_received": 307.355,
    "adjustment": 0,
    "past_due_balance": 307.36,
    "tax": 0,
    "finance_charges": 0,
    "other_charges": 0,
    "scc_charges": 0,
    "rec_charges": 0,
    "non_rec_charges": 0,
    "usage_charges": 490.94,
    "visible": true,
    "total_charges": 490.94,
    "total_amount": 490.94
  },
  "client_data": {
    "invoice_decimal_digits": 2,
    "client_name": "sip_ingress",
    "logo_url": "http://testui.denovolab.com/upload/images/ilogo.png",
    "company_info": "&lt;p&gt;Email : billing@intlcx.com, noc@intlcx.com&lt;/p&gt;\r\n\r\n&lt;p&gt;Skype: intlcx_noc&amp;nbsp;&lt;/p&gt;\r\n\r\n&lt;p&gt;website: www.intlcx.com&lt;/p&gt;\r\n\r\n&lt;p&gt;Address:&amp;nbsp;HK Office: Rooms 05-15, 13A/F, South Tower, World Finance Centre, Harbour City, 17 Canton Road, Tsim Sha Tsui, Kowloon, Hong Kong&lt;/p&gt;\r\n",
    "company_info_location": "left",
    "invoice_number": 8903,
    "invoice_date": "2021-03-13",
    "payment_due_date": "None",
    "billing_period": "2021-03-01 2021-03-13",
    "billing_info": "&lt;p&gt;US Bank Account :&lt;/p&gt;\r\n\r\n&lt;p&gt;Account Name: International Carrier Exchange Limited&lt;br /&gt;\r\nBank Name: Wells Fargo&lt;br /&gt;\r\nAccount No: 5216586643&lt;br /&gt;\r\nRouting No (Direct Deposits, Electronic Payment): 125008547&lt;br /&gt;\r\nRouting No (Wire Transfer - Domestic): 121000248&lt;/p&gt;\r\n\r\n&lt;p&gt;HK Bank Account:&lt;/p&gt;\r\n",
    "billing_info_location": "middle",
    "version": "2.0.2",
    "version_date": "2021-03-13 09:03:59"
  }
}
```

Json data may contain following data sections (parts of invoice ordered by invoicec task):
```text
'client_data',
'acc_summary', 
'payment_applied', 
'rec_charge', 
'non_rec_charge', 
'usage_charge',
'trunk_detail', 
'daily_summary', 
'jd_daily_summary', 
'jd_trunk_detail', 
'prefix_summary',
'code_name_summary',
'country_summary',
'top_100tn_summary',
'inbound_summary',
'did_charge_summary',
'did_charge_breakdown'
```
Its data mirroring data of tables included in invoice pdf.


##9. Auto invoice generation
Checks for auto invoice scheduled at every hour.
For include to generation list database "client" should follow next criterias:
```text
Client.auto_invoicing == True, 
Client.status == True, 
Client.payment_term_id is not NULL, 
Client.name is not NULL,
```
Then schedule checks if client.invoice_zone match current hour.
Next check for invoice_history and get last generated period for this client.
if history not found, first start_date will take from client creation time. 
Then check of client payment_term type:
```text
type - one of 1: 'Every', 2: 'Day of Month', 3: 'Day of Week',4: 'Some day of month', 5: 'Twice a month'
days - length of period in days or day of month or day of week (accordingly of type)  
```
Calculate next invoice end date for those payment_term.
If last invoice expired following criterias above (next invoice not overlap last one), we create new invoice.

Those settings for auto invoice will taken from "invoice_settings" table
```
pdf_tpl - invoice bank info template
tpl_number - position of template one of 0: 'bottom', 1: 'top', 2: 'middle'
company_info - company info template one of 0: 'top', 1: 'left'
company_info_location - position of company info template
send_cdr_fields - fields to include in CDR link provided in invoice email
invoice_send_mode - one of 0: 'none', 1: 'link', 2: 'attachment'
invoice_decimal_digits = Column(Integer)
```
If client's invoice_settings not assigned, same data will taked from "system_parameter" table (default invoice settings)

Here is map for including parts of invoice to the auto invoice task, based on "client" data:

```text
show_acc_summary=client.is_invoice_account_summary,
show_payment_applied=client.invoice_include_payment,
show_rec_charge=client.company_type='origination' (or for v5 if client.client_type is NULL),
show_non_rec_charge=client.company_type='origination' (or for v5 if client.client_type is NULL),
show_usage_charge=client.invoice_show_details,
show_trunk_detail=client.is_show_detail_trunk,
show_daily_summary=client.is_show_daily_usage,
show_jd_daily_summary=client.is_show_daily_usage and client.invoice_jurisdictional_detail,
show_jd_trunk_detail=client.invoice_jurisdictional_detail,
show_prefix_summary=client.is_show_total_trunk,
show_code_name_summary=client.is_show_code_name,
show_country_summary=client.is_show_country,
show_inbound_summary=client.is_show_by_date,
show_top_100tn_summary=client.is_show_code_100,
show_did_charge_breakdown=client.company_type='origination' (or for v5 if client.client_type is NULL),
show_did_charge_summary=client.company_type='origination' (or for v5 if client.client_type is NULL),
enable_email=client.email_invoice,
enable_email_with_cdr=client.invoice_send_mode_cdr in ('link', 'attachment'),
include_tax=client.include_tax,
include_scc_charges=client.is_short_duration_call_surcharge_detail,
payment_dur_date= (calculated bases on payment terms),
invoice_zone=client.invoice_zone,
```
###9.1. Auto invoice shedule checking:
Usage
```shell
curl -X GET "http://localhost:18881/inv5/v1/client/288/auto_invoice_test?date=2021-03-28" -H  "accept: application/json"
```
responce
```json
{
  "invoice_next_date": "2021-03-24",
  "invoice_cycle_days": [],
  "messages": "Client sip_ingress id 288  period 2021-03-23 2021-03-27 success",
  "invoice_next_date_of_test": "2021-03-28",
  "generated_period": [
    {
      "start_time": "2021-03-23T00:00:00",
      "end_time": "2021-03-27T23:59:59"
    }
  ],
  "invoice_cycle": "Every 1st day(s)",
  "last_invoice_date": "2021-03-23",
  "will_generate": true
}
```
##10. Alerts scheduled 
Now, the daemon will fire 7 types of alerts:
###10.1 Notify client balance
Fired when client balance go lower than limits.  

Check every 5 minute for each “active” clients’ current balance and “low
balance” trigger.  
If the “current balance” is below the “low balance”trigger, then send out an alert.

Also, there is a “number of time” that we should send in total before payment
or credit is added. Each day, the low balance should be sent once.  

The subsequent notification should be sent on 00:00:00 of the client’s GMT timezone
( default is gmt+0) Notification Setting of client:

    select notify_client_balance, low_balance_notice from client; 

To check if client is
active:

    Select status from client ;

**Enable flag: client.low_balance_notice**

**Email template: mail_tmplate.lowbalance_content**
   
###10.2. Notify zero balance

Fired when client balance below zero or below credit limit.

Check every 5 minute for each “active” clients’ current balance and “low
balance” trigger.  If the “current balance” is below 0 for prepay client type
or below  ( negative value of credit ) for postpay client type.

Also, there is a “number of time” that we should send in total before payment
or credit is added. Each day, the low balance should be sent once.  The
subsequent notification should be sent on 00:00:00 of the client’s GMT timezone
( default is gmt+0) Notification Setting of client:

    select notify_client_balance, low_balance_notice from client; 
To check if client isa ctive:

    select status from client ;
Check mode

    select mode from client ;
 * Mode = 1 (prepay)
 * Mode = 2 (postpay)

Check credit 
   
    select credit from client;

**Enable flag: client.zero_balance_notice**

**Email template: mail_tmplate.zerobalance_content**

###10.3. Daily usage summary

Notification for client od daily usage.
This notification send every day at scheduled time.

For each client who has “daily usage summary” selected, at the client’s GMT
time zone, we need to send out a daily usage summary mail.

**Enable flag: client.is_show_daily_usage**

**Email template: mail_tmplate.auto_summary_content**

###10.4. Daily balance summary
Information for clients about his balance.

For each client who has “daily balance summary” selected, at the client’s GMT time zone, 
we need to send out a daily balance summary mail.


**Enable flag: client.is_daily_balance_notification**

**Email template: mail_tmplate.auto_balance_content**

###10.5. Daily cdr delivery
Send link to download detailed information about resource usage.

For each client who has “daily CDR delivery” selected, at the client’s GMT time zone, 
we need to send out a daily CDR mail. Instead of including a  large attachment, it should be a CDR link.


**Enable flag: client.daily_cdr_generation**

**Email template: mail_tmplate.auto_cdr**

###10.6. Trunk pending suspension notice
Send notice about near future deadline of download rate limits file.

For each client, at the client’s timezone 00:00:00, we need to check if
there is any pending rate download and the deadline is to be reached within    24 hours.
If so,send this email. Select download_deadline from rate_send_log;

    select client_id , resource_id from rate_send_log_detail
    , resource where resource.resource_id = rate_send_log_detail.resource_id

    select * from rate_download_log where client_id = xx and log_detail_id = xx

** Enable flag: rate_send_log_detail.status== 5 ('Not Yet Downloaded') **

** Email templates: mail_tmplate.hour_1_content,mail_tmplate.hour_3_content,mail_tmplate.hour_24_content,mail_tmplate.hour_48_content, **


###10.7. Trunk is suspended notice
If file (item 6) not download, daemon wil send email to client and set in database block on selected resource.


For each client, at the client’s timezone 00:00:00, we need to check if
    there is any pending rate download and the deadline is passed. If so, pls
send this email AND “disable this relevant trunk.

    select download_deadline from rate_send_log; Select client_id , resource_id from rate_send_log_detail
    , resource where resource.resource_id = rate_send_log_detail.resource_id


    select * from rate_download_log where client_id = xx and log_detail_id = xx

** Enable flags:  rate_send_log.is_disable,  rate_send_log.is_email ** 
** Condition: rate_send_log_detail.status== 5 ('Not Yet Downloaded') and currend date > rate_send_log.download_deadline **

**Email templates: mail_tmplate.no_download_rate,mail_tmplate.pending_trunk**

* When "is_disable=false is_email=true" then will send  no_download_rate_content email  (Only warn about deadline riched) *
* When "is_disable=true is_email=true" then will send  pending_trunk_content email and disable trunk  *

