import os

import falcon

from . import settings,model
from .routes import ROUTES


def api_error_serializer(req, resp, exception):
    if isinstance(exception, falcon.HTTPError):
        if exception.status in [falcon.HTTP_753, falcon.HTTP_FORBIDDEN]:
            return
    else:  # pragma: no cover
        representation = exception.to_json()
        resp.body = representation
        resp.content_type = 'application/json'


def create(routes, swagger=None, json=True, yaml=True, name='default'):
    middleware = settings.get_middleware()
    for item in middleware:
        item.set_app_name(name)

    app = falcon.API(middleware=middleware)
    for tag_name, tag in routes.items():
        routes_list = tag['routes'] if isinstance(tag, dict) else tag
        for route in routes_list:
            app.add_route('{}{}'.format(settings.API_BASE_PATH, route['path']), route['resource'])

    if swagger and json:
        app.add_route('{}/swagger.json'.format(settings.API_BASE_PATH), swagger.endpoints)

    if swagger and yaml:
        app.add_route('{}/swagger.yaml'.format(settings.API_BASE_PATH), swagger.endpoints)

    app.set_error_serializer(api_error_serializer)
    return app


def create_app():
    api = falcon.API()
    api.add_route(settings.API_BASE_PATH+'/invoice/{invoice_number}', view.Invoice())
    return api

def init_version():
    from datetime import datetime
    from pytz import UTC
    from .model import VersionInformation
    from .__version__ import __date__,__minor__,__version__
    vi = VersionInformation.filter(VersionInformation.program_name == 'inv5').first()
    if not vi:
        vi = VersionInformation(program_name='inv5')

    if not (vi.minor_ver == __minor__ and vi.major_ver == __version__ and vi.build_date == __date__[0:10]):
        vi.major_ver = __version__
        vi.minor_ver = __minor__
        vi.build_date = __date__[0:10]
        vi.start_time = datetime.now(UTC)
        vi.save()

    olds = VersionInformation.filter(VersionInformation.program_name.in_(
        ['dnl_invoice_AutoInvoice',
         'dnl_invoice_REST_API',
         'dnl_invoice_server',
         'dnl_did_invoice_server. ',
         'dnl_did_invoice_server',
         'dnl_ad',
         ])).all()
    for old in olds:
        old.delete()



def get_app():
    model.init_db()
    from inv5.swagger import init
    try:
        init_version()
    except Exception as e:
        pass
    swagger = init(ROUTES,['inv5.scheme'])
    return create(ROUTES, swagger)