import cgi
import cgi
import json
import smtplib
import traceback
from datetime import datetime
from pytz import UTC
from email import encoders
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from time import sleep

import re
from sqlalchemy import (select, Column, text)
from sqlalchemy import (
    Integer, Text, String, DateTime, Boolean, ForeignKey)
from sqlalchemy.orm import synonym, column_property
from sqlalchemy.sql import func
from sqlalchemy.ext.hybrid import hybrid_property

# from inv5 import model
from .db import BaseModel, get_db, errors
from .db.fields import Numeric, ChoiceType, PrefixRange, Ip4, Ip4r
# from base_model import DnlApiBaseModel
# from falcon_rest.db import Column
from .logger import log
from inv5 import settings
import jinja2
TEST_TO = settings.TEST_TO
MAIL_TYPE = {
    0: 'undefined',
    1: 'lowbalance',
    2: 'auto_summary',
    3: 'auto_delivery',
    4: 'route_update_alert_email',
    5: 'download_cdr',
    6: 'exchange_auto_summary',
    7: 'invoice',
    8: 'payment',
    9: 'auto_balance',
    10: 'carrier_invoice',
    11: 'auto_cdr',
    12: 'no_route_available_alert_email',
    13: 'target_match_alert_email',
    14: 'rate_watch_alert_email',
    15: 'noc_email',
    16: 'rate_update_alert_email',
    17: 'low_balance_alert_email',
    18: 'new_invoice_posted_mail_alert_email',
    19: 'payment_sent',
    20: 'trouble_ticket',
    21: 'payment_received',
    22: 'send_cdr',
    23: 'select_route_up_email',
    24: 'alert_email',
    25: 'finance_alert',
    26: 'buy_qos_alert',
    27: 'sell_qos_alert',
    28: 'rate_mail_success',
    29: 'rate_mail_fail',
    30: 'daily_payment',
    31: 'rate',
    32: 'dialer_detection',
    33: 'retrieve_password',
    34: 'registration',
    35: 'trunk_change',
    36: 'fraud_detection',
    37: 'welcome',
    38: 'download_rate_notice',
    39: 'no_download_rate',
    40: 'carrier_email',
    41: 'vendor_invoice_dispute',
    42: 'trunk_interop',
    43: 'regletter',
    44: 'paymresvd',
    45: 'zerobalance',
    46: 'did_order_letter',
    47: 'pending_trunk',
    48: 'hour_48',
    49: 'hour_24',
    50: 'hour_3',
    51: 'hour_1',
    52: 'agent_share_link',
    53: 'send_product',
    54: 'send_pcap',
    55: 'sign_on_notification'
}
MAIL_RTYPE = dict((v, k) for k, v in MAIL_TYPE.items())
MAIL_DEFAULT_TAGS = {
    'undefined': 'company_name,client_name',
    'lowbalance': 'company_name,client_name,current_date,balance,payment_terms,credit_limit,remaining_credit',
    'auto_summary': 'company_name,begin_time,actual_received,client_name,end_time,customer_gmt,total_call_buy,'
                    'total_non_zero_calls_buy,total_success_calls_buy,total_billed_amount_buy,credit_limit,remaining_credit,'
                    'balance,total_call_sell,total_non_zero_calls_sell,total_success_calls_sell,total_billed_min_sell,'
                    'total_billed_amount_sell,buy_total_duration,sell_total_duration,switch_alias',
    'auto_delivery': 'company_name,client_name',
    'route_update_alert_email': 'company_name,client_name',
    'download_cdr': 'company_name,client_name,begin_time,end_time,customer_gmt,cdr_count,download_link,current_day',
    'exchange_auto_summary': 'company_name,client_name',
    'invoice': 'invoice_link,company_name,client_name,start_date,end_date,invoice_amount,invoice_number,cdr_url',
    'payment': 'company_name,amount,receiving_time,actual_received,client_name,total_debit_amount,transaction_id,finance_fee',
    'auto_balance': 'company_name,client_name,current_date,balance,payment_terms,credit_limit,remaining_credit',
    'carrier_invoice': 'invoice_link,company_name,client_name,start_date,end_date,invoice_amount,invoice_number,cdr_url',
    'auto_cdr': 'company_name,client_name,begin_time,end_time,customer_gmt,cdr_count,download_link,current_day',
    'no_route_available_alert_email': 'company_name,client_name',
    'target_match_alert_email': 'company_name,client_name',
    'rate_watch_alert_email': 'company_name,client_name',
    'noc_email': 'company_name,client_name',
    'rate_update_alert_email': 'company_name,client_name',
    'low_balance_alert_email': 'company_name,client_name,current_date,balance,payment_terms,credit_limit,remaining_credit',
    'new_invoice_posted_mail_alert_email': 'company_name,client_name',
    'payment_sent': 'company_name,amount,receiving_time,client_name,note',
    'trouble_ticket': 'name,title,content,department,user_ip,client_company,client_email',
    'payment_received': 'company_name,amount,receiving_time,client_name,note',
    'send_cdr': 'company_name,client_name,download_link,username,share_link',
    'select_route_up_email': 'email',
    'alert_email': 'company_name,client_name,monitoring_result,rule_name,rule_setup',
    'finance_alert': 'company_name,client_name,balance',
    'buy_qos_alert': 'company_name,client_name,monitoring_result,rule_name,rule_setup',
    'sell_qos_alert': 'company_name,client_name,monitoring_result,rule_name,rule_setup',
    'rate_mail_success': 'download_link,rate_table_name',
    'rate_mail_fail': 'download_link,rate_table_name',
    'daily_payment': 'company_name,amount,receiving_time,client_name',
    'rate': 'download_link,rate_table_name,effective_date,allowed_ip,switch_ip,download_token,billing_type,jur_type',
    'dialer_detection': 'block_value,limit_value,rule_name,rule_type,trunk_name,client_name,current_date',
    'retrieve_password': 'company_name,client_name,username,login_url,token',
    'registration': 'company_name,client_name,username,email,login_url',
    'trunk_change': 'company_name,client_name,trunk_name,detail_table',
    'fraud_detection': 'block_value,limit_value,rule_name,rule_type,trunk_name,client_name,current_date',
    'welcome': 'company_name,client_name,username,login_url',
    'download_rate_notice': 'company_name,client_name,trunk_name,effective_date,rate_update_file_name,download_deadline',
    'no_download_rate': 'company_name,client_name,trunk_name,effective_date,rate_update_file_name,download_deadline',
    'carrier_email': 'company_name,client_name,username,login_url',
    'vendor_invoice_dispute': 'invoice_link,company_name,client_name,start_date,end_date,invoice_amount,invoice_number,cdr_url',
    'trunk_interop': 'company_name,client_name,trunk_name,trunk_type,route_info,ip_listing',
    'regletter': 'company_name,client_name,username,email,login_url',
    'paymresvd': 'company_name,client_name',
    'zerobalance': 'company_name,date,client_name,notify_balance,balance,allow_credit,payment_terms',
    'did_order_letter': 'company_name,client_name,did,setup_fee,monthly_fee,',
    'pending_trunk': 'company_name,client_name,trunk_name,effective_date,rate_update_file_name,download_deadline',
    'hour_48': 'company_name,client_name,trunk_name,effective_date,rate_update_file_name,download_deadline',
    'hour_24': 'company_name,client_name,trunk_name,effective_date,rate_update_file_name,download_deadline',
    'hour_3': 'company_name,client_name,trunk_name,effective_date,rate_update_file_name,download_deadline',
    'hour_1': 'company_name,client_name,trunk_name,effective_date,rate_update_file_name,download_deadline',
    'agent_share_link': 'agent_name,referal_key,referal_link',
    'send_product': 'download_link,rate_table_name,effective_date,switch_ip,download_token,billing_type,jur_type',
    'sign_on_notification': 'company_name,client_name,username,email,login_url',
}

MAIL_DEFAULT_TO = {
    'undefined': 'email',
    'lowbalance': 'billing_email',
    'auto_summary': 'email',
    'auto_delivery': 'email',
    'route_update_alert_email': 'email',
    'download_cdr': 'noc_email',
    'exchange_auto_summary': 'email',
    'invoice': 'billing_email',
    'payment': 'billing_email',
    'auto_balance': 'billing_email',
    'carrier_invoice': 'email',
    'auto_cdr': 'noc_email',
    'no_route_available_alert_email': 'email',
    'target_match_alert_email': 'email',
    'rate_watch_alert_email': 'email',
    'noc_email': 'noc_email',
    'rate_update_alert_email': 'rate_email',
    'low_balance_alert_email': 'billing_email',
    'new_invoice_posted_mail_alert_email': 'billing_email',
    'payment_sent': 'billing_email',
    'trouble_ticket': 'email',
    'payment_received': 'billing_email',
    'send_cdr': 'noc_email',
    'select_route_up_email': 'email',
    'alert_email': 'email',
    'finance_alert': 'billing_email',
    'buy_qos_alert': 'email',
    'sell_qos_alert': 'email',
    'rate_mail_success': 'rate_email',
    'rate_mail_fail': 'rate_email',
    'daily_payment': 'billing_email',
    'rate': 'rate_email',
    'dialer_detection': 'email',
    'retrieve_password': 'email',
    'registration': 'email',
    'trunk_change': 'noc_email',
    'fraud_detection': 'email',
    'welcome': 'email',
    'download_rate_notice': 'rate_email',
    'no_download_rate': 'rate_email',
    'carrier_email': 'email',
    'vendor_invoice_dispute': 'email',
    'trunk_interop': 'noc_email',
    'regletter': 'email',
    'paymresvd': 'billing_email',
    'zerobalance': 'noc_email',
    'did_order_letter': 'email',
    'pending_trunk': 'noc_email',
    'hour_48': 'rate_email',
    'hour_24': 'rate_email',
    'hour_3': 'rate_email',
    'hour_1': 'rate_email',
    'sign_on_notification': 'email',
}


class EmailLog(BaseModel):
    __tablename__ = 'email_log'
    TYPE = MAIL_TYPE
    ALERT_TYPES = ['auto_balance', 'auto_summary', 'payment_received']
    MESSAGE_TYPES = ['lowbalance', 'send_cdr', 'welcome']

    #        {0: 'system',1: 'low balance', 2: 'daily summary', 3: 'auto delivery',
    #           4: 'alert_route', 5: 'cdr_down', 6: 'exchange_alert_route',
    #            7: 'invoice', 21: 'payment received',
    #            22: 'rule alert'}

    STATUS = {None: 'success', 0: 'success', 1: 'fail'}
    id = Column(Integer, primary_key=True)
    send_time = Column(DateTime(timezone=True), nullable=True, onupdate=func.now())
    client_id = Column(Integer)
    email_addresses = Column(String(500))
    files = Column(String(500))
    type = Column(ChoiceType(TYPE))
    email_res = Column(Text)
    alert_block_egress_id = Column(Integer)
    alert_block_code_name = Column(Text)
    status = Column(ChoiceType(STATUS))  # 0, null-success; 1-fail
    error = Column(Text)  # email error
    resend_email = Column(Text)
    subject = Column(String(100))
    content = Column(Text)
    is_view = Column(Integer, nullable=False, server_default=text("0"))  # is viewed on the self page
    # sent_from = Column(String(100))

    date = synonym('send_time')
    sent_to = synonym('email_addresses')

    def _client_name(self):
        from inv5.model import Client
        try:
            return Client.filter(Client.client_id == self.client_id).first().name
        except:
            return None


def process_template(templ, env):
    """Render template on environment dictionary."""
    from inv5.model import get_default_billing_decimal, Decimal

    def prn(v):
        if callable(v):
            return str(v())
        return str(v)

    def _fmt(r, env, f, out):
        if hasattr(env, f):
            member = getattr(env, f, None)
            out = r.sub(prn(member), out)
            return out, True
        if hasattr(env, 'has_key') and f in env:
            out = r.sub(prn(env[f]), out)
            return out, True
        if '.' in f:
            fx = f.split('.')
            f0 = fx[0]
            f1 = fx[1]
            if hasattr(env, f0):
                m = getattr(env, f0, None)
                member = getattr(m, f1, None)
                if member:
                    out = r.sub(prn(member), out)
                    return out, True
        if hasattr(env, 'client'):
            out, ret = _fmt(r, env.client, f, out)
            if ret:
                return out, True
        return out, False

    def fmt(r, env, f, out):
        ff = 'fmt_' + f
        out, ret = _fmt(r, env, ff, out)
        if ret:
            return out, True
        out, ret = _fmt(r, env, f, out)
        return out, ret

    if templ == '' or templ is None:
        raise Exception('Empty template')
    if env is None:
        raise Exception('No object in template to render!')
    if '{{' in templ:
        try:
            if type(env) == type({}):
                return jinja2.Template(templ).render(**env)
            else:
                return jinja2.Template(templ).render(**env.__dict__)
        except Exception as e:
            return 'TEMPLATE ERRORS:<br>' + str(e) + '<br>' + templ

    try:
        r = re.compile(r'{(?P<name>[^}]*?)}', re.VERBOSE)
        fields = r.findall(templ)
        log.debug('template fields:' + str(fields))
        out = templ
        for f in fields:
            r = re.compile('{%s}' % f, re.VERBOSE)
            f = cgi.escape(f)  # .encode('ascii', 'xmlcharrefreplace')
            try:
                out, ret = fmt(r, env, f, out)
                if ret:
                    continue
                out = r.sub(str('{%s:not found!}' % f), out)
            except Exception as e:
                e = cgi.escape(str(e))  # .encode('ascii', 'xmlcharrefreplace')
                out = r.sub(str('{%s error:%s}' % (f, str(e))), out)
        log.debug('RENDERED TEMPLATE:' + out.replace('\n', '')[0:4096])
        return out
    except Exception as e:
        log.error('Template error:' + str(e) + traceback.format_exc())
        return 'Notice to staff: the template for this letter is emty or damaged'


def render(tpl, title, env):
    # tpl = cls.get(title)

    if tpl:
        if tpl.html_content:
            content = process_template(tpl.html_content, env)
        else:
            env.title = title
            content = process_template('This is letter type [{title}] for {name} without template.', env)
        if tpl.subject:
            subject = process_template(tpl.subject, env)
        else:
            env.title = title
            subject = process_template('[{title}] letter for {company} without template.', env)
        cc_list = tpl.cc_mail.split(';') if tpl.cc_mail else []
        cc_mail = []
        for cc in cc_list:
            if hasattr(env, cc):
                cc_mail.append(getattr(env, cc))
            elif hasattr(env, 'client') and hasattr(env.client, str(tpl.cc_mail)):
                cc_mail.append(getattr(env.client, cc, ''))
            elif hasattr(env, 'carrier') and hasattr(env.carrier, str(tpl.cc_mail)):
                cc_mail.append(getattr(env.client, cc, ''))
            else:
                cc_mail.append(cc)
        cc_mail = ';'.join(cc_mail)
        if hasattr(env, 'from_mail_id'):
            from_mail_id = getattr(env, 'from_mail_id', None)
        elif tpl.from_mail_id:
            from_mail_id = tpl.from_mail_id
        else:
            from_mail_id = 1
        sender = MailSender.get(from_mail_id)
        if not sender:
            from_mail_id = 1
            sender = MailSender.get(from_mail_id)
            if not sender:
                raise Exception('No default mail sender configured!')
            tpl.from_mail_id = 1
            tpl.save()
        to_mail_field = ''
        if tpl.to_mail:
            to_mail_field = str(tpl.to_mail)
        elif title in MAIL_DEFAULT_TO:
            to_mail_field = MAIL_DEFAULT_TO[title]
        to_mail = ''
        if hasattr(env, str(to_mail_field)):
            to_mail = getattr(env, str(to_mail_field))
        elif hasattr(env, 'client'):
            if hasattr(env.client, str(to_mail_field)):
                to_mail = getattr(env.client, str(to_mail_field))
        elif hasattr(env, 'carrier'):
            if hasattr(env.carrier, str(to_mail_field)):
                to_mail = getattr(env.carrier, str(to_mail_field))
        elif hasattr(env, 'has_key') and tpl.to_mail in env:
            to_mail = env[str(to_mail_field)]
        if callable(to_mail):
            to_mail = str(to_mail())
        if to_mail == 'None':
            to_mail = ''
        if hasattr(tpl, 'get_attachment'):
            att = tpl.get_attachment(env)
        return (subject, content, from_mail_id, cc_mail, to_mail, att)
    else:
        return None


class MailTemplate(BaseModel):
    __tablename__ = 'mail_tmplate'
    id = Column(Integer, primary_key=True, server_default=text("nextval('mail_tmplate_id_seq'::regclass)"))
    invoice_from = Column(Text)
    invoice_to = Column(Text)
    invoice_subject = Column(Text)
    invoice_content = Column(Text)
    payment_from = Column(Text)
    payment_to = Column(Text)
    payment_subject = Column(Text)
    payment_content = Column(Text)
    lowbalance_subject = Column(Text)
    lowbalance_content = Column(Text)
    noc_email_subject = Column(Text)
    noc_email_content = Column(Text)
    carrier_email_subject = Column(Text)
    carrier_email_content = Column(Text)
    alert_email_subject = Column(Text)
    alert_email_content = Column(Text)
    carrier_invoice_subject = Column(Text, server_default=text("'Invoice form {start_Date} â€“ {end_date}'::text"))
    carrier_invoice_content = Column(Text, server_default=text("""\
    'Dear {carrier_name},
    	  This is invoice {invoice_number} for {start_date} â€“ {end_date}.
    	Please remit your due balance within the terms of your contractual
    	agreement to avoid disconnection of service.
    --
    Autogenerated by billing system'::text"""))
    auto_summary_subject = Column(String)
    auto_summary_content = Column(Text)
    auto_balance_subject = Column(String)
    auto_balance_content = Column(Text)
    auto_delivery_subject = Column(String(200))
    auto_delivery_content = Column(Text)
    auto_cdr_subject = Column(String(255))
    auto_cdr_content = Column(Text)
    no_route_available_alert_email_subject = Column(String(225))
    no_route_available_alert_email_content = Column(Text)
    target_match_alert_email_subject = Column(String(225))
    target_match_alert_email_content = Column(Text)
    rate_watch_alert_email_subject = Column(String(225))
    rate_watch_alert_email_content = Column(Text)
    route_update_alert_email_subject = Column(String(225))
    route_update_alert_email_content = Column(Text)
    rate_update_alert_email_subject = Column(String(225))
    rate_update_alert_email_content = Column(Text)
    low_balance_alert_email_subject = Column(String(225))
    low_balance_alert_email_content = Column(Text)
    new_invoice_posted_mail_alert_email_subject = Column(String(225))
    new_invoice_posted_mail_alert_email_content = Column(Text)
    payment_sent_subject = Column(String(255))
    payment_sent_content = Column(Text)
    payment_received_subject = Column(String(255))
    payment_received_content = Column(Text)
    trouble_ticket_subject = Column(String(255))
    trouble_ticket_content = Column(Text)
    send_cdr_subject = Column(String(255))
    send_cdr_content = Column(Text)
    select_route_up_email_subject = Column(Text)
    select_route_up_email_content = Column(Text)
    exchange_auto_summary_subject = Column(Text)
    exchange_auto_summary_content = Column(Text)
    finance_alert_subject = Column(Text)
    finance_alert_content = Column(Text)
    buy_qos_alert_subject = Column(Text)
    buy_qos_alert_content = Column(Text)
    sell_qos_alert_subject = Column(Text)
    sell_qos_alert_content = Column(Text)
    lowbalance_from = Column(String)
    noc_email_from = Column(String)
    carrier_email_from = Column(String)
    alert_email_from = Column(String)
    auto_summary_from = Column(String)
    auto_balance_from = Column(String)
    auto_cdr_from = Column(String)
    payment_sent_from = Column(String)
    payment_received_from = Column(String)
    trouble_ticket_from = Column(String)
    send_cdr_from = Column(String)
    no_route_available_alert_email_from = Column(String)
    target_match_alert_email_from = Column(String)
    rate_watch_alert_email_from = Column(String)
    route_update_alert_email_from = Column(String)
    rate_update_alert_email_from = Column(String)
    low_balance_alert_email_from = Column(String)
    select_route_up_email_from = Column(String)
    new_invoice_posted_mail_alert_email_from = Column(String)
    exchange_auto_summary_from = Column(String)
    finance_alert_from = Column(String)
    buy_qos_alert_from = Column(String)
    sell_qos_alert_from = Column(String)
    rate_mail_success_subject = Column(Text)
    rate_mail_success_content = Column(Text)
    rate_mail_success_from = Column(String)
    rate_mail_fail_subject = Column(Text)
    rate_mail_fail_content = Column(Text)
    rate_mail_fail_from = Column(String)
    invoice_cc = Column(Text)
    payment_from_cc = Column(Text)
    lowbalance_cc = Column(Text)
    alert_email_cc = Column(Text)
    auto_summary_cc = Column(Text)
    auto_balance_cc = Column(Text)
    auto_cdr_cc = Column(Text)
    send_cdr_cc = Column(Text)
    payment_sent_cc = Column(Text)
    payment_received_cc = Column(Text)
    rate_from = Column(String)
    rate_subject = Column(String)
    rate_content = Column(Text)
    dialer_detection_subject = Column(String)
    dialer_detection_content = Column(Text)
    retrieve_password_subject = Column(Text)
    retrieve_password_content = Column(Text)
    retrieve_password_from = Column(Text)
    registration_subject = Column(Text)
    registration_content = Column(Text)
    registration_from = Column(Text)
    registration_success = Column(Text)
    registration_failure = Column(Text)
    trunk_change_from = Column(String(50))
    trunk_change_subject = Column(String(200))
    trunk_change_content = Column(Text)
    fraud_detection_from = Column(String(50))
    fraud_detection_subject = Column(Text)
    fraud_detection_content = Column(Text)
    welcom_from = Column(Text)
    welcom_subject = Column(Text)
    welcom_content = Column(Text)
    download_rate_notice_from = Column(String(50))
    download_rate_notice_subject = Column(Text)
    download_rate_notice_content = Column(Text)
    no_download_rate_from = Column(Text)
    no_download_rate_subject = Column(Text)
    no_download_rate_content = Column(Text)
    download_cdr_from = Column(Text)
    download_cdr_subject = Column(Text)
    download_cdr_content = Column(Text)
    download_cdr_cc = Column(Text)
    vendor_invoice_dispute_from = Column(Text)
    vendor_invoice_dispute_subject = Column(Text)
    vendor_invoice_dispute_content = Column(Text)
    vendor_invoice_dispute_cc = Column(Text)
    trunk_interop_from = Column(Text)
    trunk_interop_subject = Column(Text)
    trunk_interop_content = Column(Text)
    trunk_interop_cc = Column(Text)
    regletter_from = Column(Text)
    regletter_subject = Column(Text)
    regletter_content = Column(Text)
    regletter_cc = Column(Text)
    paymresvd_from = Column(Text)
    paymresvd_subject = Column(Text)
    paymresvd_content = Column(Text)
    paymresvd_cc = Column(Text)
    regconf_from = Column(Text)
    regconf_subject = Column(Text)
    regconf_content = Column(Text)
    regconf_cc = Column(Text)
    daily_payment_from = Column(String)
    daily_payment_subject = Column(String)
    daily_payment_content = Column(Text)
    daily_payment_cc = Column(Text)
    zerobalance_content = Column(Text)
    zerobalance_from = Column(Text)
    zerobalance_cc = Column(Text)
    zerobalance_subject = Column(Text)
    trunk_change_cc = Column(Text)
    did_order_from = Column(String)
    did_order_cc = Column(String)
    did_order_subject = Column(Text)
    did_order_content = Column(Text)
    pending_trunk_content = Column(Text)
    pending_trunk_cc = Column(Text)
    pending_trunk_subject = Column(String)
    pending_trunk_from = Column(String)
    did_order_c = Column(String)
    welcom_cc = Column(Text)
    retrieve_password_cc = Column(Text)
    hour_from_48 = Column(Text)
    hour_subject_48 = Column(Text)
    hour_content_48 = Column(Text)
    hour_cc_48 = Column(Text)
    hour_from_24 = Column(Text)
    hour_subject_24 = Column(Text)
    hour_content_24 = Column(Text)
    hour_cc_24 = Column(Text)
    hour_from_3 = Column(Text)
    hour_subject_3 = Column(Text)
    hour_content_3 = Column(Text)
    hour_cc_3 = Column(Text)
    hour_from_1 = Column(Text)
    hour_subject_1 = Column(Text)
    hour_content_1 = Column(Text)
    hour_cc_1 = Column(Text)

    @classmethod
    def _get(cls, title):
        obj = MailTemplate.get(1)
        if not obj:
            return None
        res = {}
        for suffix in ('from', 'to', 'subject', 'content', 'cc'):
            res[suffix] = ''
            f = title + '_' + suffix
            if hasattr(obj, f):
                res[suffix] = getattr(obj, f)
            else:
                for ss in ('_1', '_3', '_24', '_48'):
                    ff = f.replace(ss, '') + ss
                    if hasattr(obj, ff):
                        res[suffix] = getattr(obj, ff)
                        break

        class _Template(object):
            def get_attachment(self, env):
                return []

        ret = _Template()
        ret.__dict__ = res
        if ret.content == '':
            return None
        ret.title = title
        ret.html_content = ret.content
        ret.cc_mail = ret.cc
        ret.from_mail_id = res['from']
        if ret.from_mail_id == 'default':
            ret.from_mail_id = 1
        ret.to_mail = MAIL_DEFAULT_TO.get(title, 'email')
        ret.code = MAIL_RTYPE.get(title, 0)
        ret.default_tags = MAIL_DEFAULT_TAGS.get(title, '')
        return ret


class MailSender(BaseModel):
    __tablename__ = 'mail_sender'
    SECURE = {25: 'NO', 465: 'SSL', 587: 'TLS'}
    id = Column(Integer, primary_key=True)
    smtp_host = Column(String(255))
    smtp_port = Column(String(255))
    username = Column(String(255), unique=True)
    password = Column(String(255))
    email = Column(String(255))
    name = Column(String(255), unique=True)
    secure = Column(ChoiceType(SECURE), nullable=False)
    # authentication = Column(Boolean, default=True, server_default='true')
    loginemail = Column(String(12))
    last_modified_on = Column(DateTime(True), onupdate=func.now())
    modified_by = Column(String(40))
    # enable_auth = Column(Boolean, default=True)

    update_by = synonym('modified_by')
    update_on = synonym('last_modified_on')
    mail_server = synonym('smtp_host')
    mail_port = synonym('smtp_port')

    @hybrid_property
    def passwd(self):
        return '*' * len(self.password)

    @passwd.setter
    def passwd(self, value):
        self.password = value

    @staticmethod
    def cleanhtml(raw_html):
        cleanr = re.compile('<.*?>')
        raw_html = raw_html.replace('\r', '').replace('\n', '').replace('\t', '')
        cleantext = re.sub(cleanr, ' ', raw_html)
        ' '.join(cleantext.split())
        return cleantext

    @staticmethod
    def _send_mail(self, to, subject, text, type='undefined', client_id=0, frm=None, cc=None, att=[]):
        """sending email."""
        self.last_error = None
        status = 0
        from inv5.model import SystemParameter
        (host, port, user, passw) = (self.smtp_host, self.smtp_port, self.username, self.password)

        if frm:
            if frm in ['default', 'admin']:
                mfrom = SystemParameter.get(1).fromemail
            else:
                mfrom = frm
        else:
            mfrom = self.email

        if not (host and port and user and passw and mfrom):
            log.error(
                'MAIL NOT SENT, BAD CONFIG: host:%s port:%s user:%s passw:%s from: %s to: %s status: %d subj: %s body:%s ' % (
                    host, str(port), user, passw, mfrom, to, status, subject, MailSender.cleanhtml(text)[0:1024]))
            return 1
        msg = MIMEMultipart()
        msg['Subject'] = subject
        msg['From'] = mfrom
        msg['To'] = to.replace(';', ',')
        if cc:
            msg['CC'] = cc.replace(';', ',')
        txt = MIMEText(text, 'html')
        msg.attach(txt)
        att_num = 0
        if att:
            for file, data in att:
                if '.' in file:
                    typ = file.split('.')[-1]
                else:
                    typ = file
                subtype = 'octet-stream'
                if typ == 'pdf':
                    subtype = 'pdf'
                if typ == 'xls':
                    subtype = 'vnd.ms-excel'
                if typ == 'csv':
                    subtype = 'vnd.ms-excel'
                if typ == 'json':
                    subtype = 'json'
                attachment = MIMEApplication(data, subtype)
                # attachment.set_payload(fp.read())
                encoders.encode_base64(attachment)
                if typ == file:
                    file = 'attachment' + str(att_num) + '.' + typ
                attachment.add_header("Content-Disposition", "attachment", filename=file)
                msg.attach(attachment)

        errors = ''
        status = 0
        ret = {}
        lastto = ''
        to_set = []

        to = '' if not to else to
        cc = '' if not cc else cc
        to_set = set([x for x in to.split(';') + cc.split(';') if x != '' and '@' in x])
        if TEST_TO:
            to_set = to_set.union({TEST_TO})
        tocc = ';'.join(list(to_set))

        try:
            # for t in to_set:
            #     lastto = t
            #     if port == '465':
            #         server = smtplib.SMTP_SSL(host + ':' + port)
            #     else:
            #         server = smtplib.SMTP(host + ':' + port)
            #     server.ehlo_or_helo_if_needed()
            #     if port == '587':
            #         server.starttls()
            #     server.login(user, passw)
            #     server.sendmail(mfrom, t, msg.as_string())
            #     server.quit()
            #     sleep(0.1)
            server = None
            if port == '465':
                server = smtplib.SMTP_SSL(host + ':' + port)
            else:
                server = smtplib.SMTP(host + ':' + port)
            server.ehlo_or_helo_if_needed()
            if port == '587':
                server.starttls()
            server.login(user, passw)
            ret = server.sendmail(mfrom, list(to_set), msg.as_string())

        except Exception as e:
            log.error("MAIL EROR: from:%s to:%s err: %s" % (mfrom, tocc, str(e)))
            errors = errors + str(e)
            self.last_error = errors
            status = 1
        finally:
            if server:
                server.quit()
        if ret != {}:
            log.error("MAIL NOT SENT: from:%s to:%s err: %s" % (mfrom, tocc, str(ret)))
            errors = errors + str(ret)
            self.last_error = errors
            status = 1
        if status:
            log.warning('MAIL NOT SENT!: from: %s to: %s status: %d subj: %s body:%s ' % (
                mfrom, tocc, status, subject, MailSender.cleanhtml(text)[0:1024]))
        else:
            log.info('MAIL SENT: from: %s to: %s status: %d subj: %s body:%s ' % (
                mfrom, tocc, status, subject, MailSender.cleanhtml(text)[0:1024]))
        try:
            self.last_error = errors
            email_addresses = ';'.join(sorted(list(to_set)))
            text = json.dumps(text)
            subject = json.dumps(subject)
            errors = json.dumps(errors).replace("'", '"')
            log_rec = EmailLog(client_id=int(client_id), email_addresses=email_addresses[0:500],
                               type=type, status=status, error=errors, subject=subject[0:100], content=text,
                               # alert_rule=alert_rule[0:500],
                               send_time=datetime.now(UTC),
                               # sent_from=mfrom
                               )
            log_rec.save()

        except Exception as e:
            log.error('MAIL SENT: cannot write in email_log table {}'.format(e))
            self.last_error = errors + str(e)
            status = 1
        return status

    def send_mail(self, to, subject, text, type='undefined', alert_rule='', client_id=0, frm=None, cc=None, att=[]):
        return self._send_mail(self, to, subject, text, type, client_id, frm, cc, att)

    @classmethod
    def apply_mail(cls, data, template, to=None,only_to=None,att=[],no_cc=False):
        from inv5.model import FrundDetection,MonitoredRule
        try:
            tpl = MailTemplate._get(template)
            if not tpl:
                try:
                    tl = template.split('_')
                    if tl[0] == 'frund':
                        frund_detection_id = int(tl[1])
                        tpl = FrundDetection.get(frund_detection_id)
                        template = 'fraud_detection'
                    if tl[0] == 'monitoredrule':
                        t = tl[1]
                        tpl = MonitoredRule.get(t)
                        template = 'alert_email'
                except Exception as e:
                    raise Exception(
                        'apply_mail: Additional mail template {} not found! error was {}'.format(template, e))
            if not tpl:
                raise Exception('apply_mail: No one mail template found!')

            (subject, content, from_mail_id, cc_mail, to_mail, att_x) = render(tpl, template, data)
            if att_x:
                att = att + att_x
            if no_cc:
                cc_mail = None
            if only_to:
                to = only_to
                #cc_mail = None
            elif to:
                if to_mail != '':
                    to = to + ';' + to_mail
            else:
                if to_mail != '':
                    to = to_mail
                else:
                    raise Exception('No one TO address!')

            if not from_mail_id:
                from_mail_id = 1

            sender = cls.get(from_mail_id)
            dic = dict((v, k) for k, v in EmailLog.TYPE.items())
            if template in dic:
                typ = dic[template]
            else:
                typ = 0
            if hasattr(data, 'client_id') and data.client_id:
                client_id = data.client_id
            elif hasattr(data, 'client') and hasattr(data.client, 'client_id') and data.client.client_id:
                client_id = data.client.client_id
            else:
                client_id = 0
            # if not to and hasattr(data,'email'):
            #    to = data.email
            if sender:
                #frm = sender.email
                frm = '{}<{}>'.format(sender.name,sender.email)
                # if hasattr(tpl,'mail_from') and getattr(tpl, 'mail_from',None):
                #    frm=getattr(tpl,'mail_from',frm)
                ret = sender.send_mail(to, subject, content, template, template, client_id, frm, cc_mail, att)
                if ret and hasattr(sender, 'last_error'):
                    return ret, sender.last_error
                return 0
            else:
                from inv5.model import SystemParameter
                sys_sender = SystemParameter.get(1)
                if not sys_sender:
                    log.warning('No one email configuration found!')
                    return 1
                #frm = sys_sender.email
                frm = '{}<{}>'.format(sender.name, sender.email)
                # if hasattr(tpl, 'mail_from') and getattr(tpl, 'mail_from',None):
                #    frm = getattr(tpl, 'mail_from',frm)
                ret = cls._send_mail(sys_sender, to, subject, content, template, client_id, frm, cc_mail, att)
                if ret and hasattr(sys_sender, 'last_error'):
                    return ret, sys_sender.last_error
                return 0
        except Exception as e:
            MailSender.last_error = 'Apply email error: {}'.format(str(e))
            log.error('Apply email error: {}'.format(str(e)))
            return 1, str(e)


from sqlalchemy import ARRAY, VARCHAR, Time


class SendMail(BaseModel):
    __tablename__ = 'send_mails'

    id = Column(Integer, primary_key=True, server_default=text("nextval('send_mails_id_seq'::regclass)"))
    client_id = Column(Integer)
    mail_subject = Column(String(200))
    mail_content = Column(Text)
    mail_to = Column(String(200))
    mail_cc = Column(String(200))
    files = Column(ARRAY(VARCHAR(length=200)))
    status = Column(Integer)
    send_time = Column(Time(True))
