from marshmallow import validate
from marshmallow.fields import Bool, Str, Decimal, Int, Nested, DateTime, Float, List, UUID, Dict

from inv5 import model
from inv5.schemes.schemes import SuccessScheme
#from .schemes.scheme_choice import Choice
from inv5.scheme import BaseModelScheme,Emails
from inv5.responses import ObjectCreatedResponse
from inv5.scheme import IP_REGEXP, NAME_REGEXP, PREFIX_REGEXP, TIME_REGEXP, ValidationError,_valid

FIELDS={0:'time',1:'release_cause',2:'start_time_of_date',3:'answer_time_of_date',4:'release_tod',5:'minutes_west_of_greenwich_mean_time',6:'release_cause_from_protocol_stack',7:'binary_value_of_release_cause_from_protocol_stack',8:'first_release_dialogue',9:'origination_source_number',10:'origination_source_host_name',11:'origination_destination_number',12:'origination_destination_host_name',13:'origination_call_id',14:'origination_remote_payload_ip_address',15:'origination_remote_payload_udp_address',16:'origination_local_payload_ip_address',17:'origination_local_payload_udp_address',18:'origination_codec_list',19:'termination_source_number',20:'termination_source_host_name',21:'termination_destination_number',22:'termination_destination_host_name',23:'termination_call_id',24:'termination_remote_payload_ip_address',25:'termination_remote_payload_udp_address',26:'termination_local_payload_ip_address',27:'termination_local_payload_udp_address',28:'termination_codec_list',29:'final_route_indication',30:'routing_digits',31:'call_duration',32:'pdd',33:'ring_time',34:'callduration_in_ms',35:'conf_id',36:'call_type',37:'ingress_id',38:'ingress_client_id',39:'ingress_client_rate_table_id',40:'ingress_client_currency_id',41:'ingress_client_rate',42:'ingress_client_currency',43:'ingress_client_bill_time',44:'ingress_client_bill_result',45:'ingress_client_cost',46:'egress_id',47:'egress_rate_table_id',48:'egress_rate',49:'egress_cost',50:'egress_bill_time',51:'egress_client_id',52:'egress_client_currency_id',53:'egress_client_currency',
        #54:'egress_six_seconds',
        55:'egress_bill_minutes',56:'egress_bill_result',57:'ingress_bill_minutes',58:'ingress_dnis_type',59:'ingress_rate_type',60:'lrn_dnis',61:'egress_dnis_type',62:'egress_rate_type',63:'item_id',64:'translation_ani',65:'ingress_rate_id',66:'egress_rate_id',67:'orig_code',68:'orig_code_name',69:'orig_country',70:'term_code',71:'term_code_name',72:'term_country',73:'ingress_rate_effective_date',74:'egress_rate_effective_date',75:'egress_erro_string',76:'order_id',77:'order_type',78:'is_final_call',79:'egress_code_asr',80:'egress_code_acd',81:'static_route',82:'dynamic_route',83:'route_plan',84:'route_prefix',85:'orig_delay_second',86:'term_delay_second',87:'orig_call_duration',88:'trunk_type',89:'origination_profile_port',90:'termination_profile_port',91:'o_trunk_type2',92:'o_billing_method',93:'t_trunk_type2',94:'t_billing_method',95:'campaign_id',96:'agent_id',97:'agent_rate',98:'agent_cost',99:'orig_jur_type',100:'term_jur_type',101:'ring_epoch',102:'end_epoch',103:'paid_user',104:'rpid_user',105:'timeout_type',106:'q850_cause'
    ,107:'q850_cause_string'
        }
FIELD_MAP=dict((v, k) for k, v in FIELDS.items())


class CdrQueryScheme(BaseModelScheme):
    fields = List(Str(validate=validate.OneOf(list(FIELD_MAP.keys()))))
    start_time = DateTime()
    end_time = DateTime()

    class Meta:
        model = model.Resource
        fields = ('start_time', 'end_time', 'fields')


def valid_cdr_detail_fields(value):
    try:
        fields = value.split(',')
        for f in fields:
            if not hasattr(model.CdrReportDetail, f):
                raise ValidationError('invalid field {}'.format(f))
    except:
        raise ValidationError('invalid fields')


class CdrReportDetailGetScheme(BaseModelScheme):
    report_time = DateTime()
    ingress_rate_table_id = Int(allow_none=True)

    # for query
    # fields = List(Str(validate=validate.OneOf(list(FIELD_MAP.keys()))))
    field = Str(validate=valid_cdr_detail_fields)
    start_time = Str()
    end_time = Str()
    release_cause = Int()
    ingress_id = Str()
    egress_id = Str()

    class Meta:
        model = model.CdrReportDetail
        fields = ()
        query_fields = ('start_time', 'end_time', 'field', 'release_cause', 'ingress_id', 'egress_id')

class CdrReportDetailEarliestGetScheme(BaseModelScheme):
    report_time = DateTime
    class Meta:
        model = model.CdrReportDetail
        fields = ()
        query_fields = ()

def valid_cdr_fields(value):
    try:
        fields = value.split(',')
        for f in fields:
            if not hasattr(model.ClientCdr, f):
                raise ValidationError('invalid field {}'.format(f))
    except:
        raise ValidationError('invalid field')


class CdrReportGetScheme(BaseModelScheme):
    # report_time = DateTime()
    # start = Int()
    # count = Int()
    start_time = Int()
    end_time = Int()
    field = Str(validate=valid_cdr_detail_fields)
    format = Str()
    tz = Int()
    call_tz = Int()
    non_zero = Int()
    human_readable = Int()

    call_duration_min = Int()
    call_duration_max = Int()
    egress_cost_min = Float()
    egress_cost_max = Float()
    ingress_client_cost_min = Float()
    ingress_client_cost_max = Float()
    ingress_name = Str()
    egress_name = Str()
    ingress_client_name = Str()
    egress_client_name = Str()
    egress_rate_table_name = Str()
    dynamic_route_name = Str()
    static_route_name = Str()
    egress_trunk_trace = Str()

    release_cause_name = Str()
    trunk_type2 = Str()
    timezone_name = Str()

    egress_rate_type_name = Str()
    ingress_rate_type_name = Str()
    ingress_dnis_type_name = Str()
    orig_jur_type_name = Str()
    term_jur_type_name = Str()
    route_plan_name = Str()


    time = Str()
    start_time_of_date = Str()
    end_epoch = Str()
    answer_time_of_date = Str()
    release_tod = Str()
    ingress_rate_effective_date = Str()
    egress_rate_effective_date = Str()

    class Meta:
        model = model.ClientCdr
        fields = ()
        query_fields = ('start_time', 'end_time', 'field', 'tz', 'human_readable', 'trunk_type2', 'non_zero')


class ObjectCreatedRequestId(SuccessScheme):
    request_id = Str()


# class CdrAsyncTaskSchemeGet(BaseModelScheme):
#     class Meta:
#         model = model.CdrAsyncTask
#         fields = ()


class CdrAsyncTaskScheme(CdrReportGetScheme):
    class Meta:
        model = model.CdrAsyncTask
        # fields = ()
        exclude = ('request_id', 'request_client_id', 'filter', 'download_link',
                   'expiration_time', 'job_start_time', 'job_end_time', 'msg', 'progress', 'count', 'size',
                   'status', 'email', 'orig_file', 'mail_status', 'method', 'type')

    @classmethod
    def get_object_created_response(cls, **kwargs):
        kwargs['data_key'] = 'request_id'
        kwargs['scheme'] = ObjectCreatedRequestId
        return ObjectCreatedResponse(**kwargs)


class CdrAsyncTaskSchemeGet(BaseModelScheme):
    start_time = Int()
    end_time = Int()
    client_id = Int()

    class Meta:
        model = model.CdrAsyncTask
        search_fields = ('start_time', 'end_time', 'job_id', 'job_start_time')
        query_fields = ('job_start_time_gt', 'job_start_time_lt')


class CdrAsyncTaskEmailScheme(BaseModelScheme):
    # request_id = Str(validate=[validate.Length(max=16),lambda value:_valid_unique('CdrDownloadTask','request_id',value)])
    request_id = Str(validate=[validate.Length(max=16), lambda value: _valid('CdrAsyncTask', 'request_id', value)])
    email = Emails()
    client_id = Int(allow_none=True, validate=lambda value: _valid('Client', 'client_id', value))
    direct = Nested('SendCdrDirectScheme', allow_none=True)

    class Meta:
        model = model.CdrAsyncTask
        fields = ('email', 'direct', 'client_id')


def validate_group(value):
    try:
        fields = value.split(',')
        for f in fields:
            if not hasattr(model.ClientCdr, f):
                raise ValidationError('invalid field {}'.format(f))
    except:
        raise ValidationError('invalid group')


def validate_step(value):
    if value in ['hour', 'day', 'week', 'month', 'year', 'all']:
        return True
    intv = int(value)
    return True


class CdrReportGroppedRequestScheme(BaseModelScheme):
    # for query
    # fields = List(Str(validate=validate.OneOf(list(FIELD_MAP.keys()))))
    # fields = Str(validate=valid_cdr_fields)
    start_time = Str(allow_none=False, required=True, description='search from specified time (allow negative values)')
    end_time = Str(allow_none=False, required=True,
                   description="search till time (end_time > start_time). Default: 0 do not limit search by time")
    method = Str(validate=validate.OneOf(['max', 'min', 'avrg', 'total']), default='total', allow_none=False,
                 required=True, description="'max', 'min', 'avrg', 'total'")
    # group = Str(validate=valid_cdr_fields)
    step = Str(validate=validate_step, default='all',
               description="granularity: 'hour', 'day', 'week', 'month', 'year', 'all', or amount of minutes.")
    sort = Str(description="sort results by field (allow multiple sort fields delimited by ','")
    order = Str(
        description="sorting order: 'asc', 'desc'. Default: descending order. Mandatory be multiple for multiple 'order'")
    is_hide_unathorized = Bool()
    class Meta:
        model = model.CdrReportDetail
        search_fields = ('start_time', 'end_time', 'method', 'step', 'sort', 'order','is_hide_unathorized')


class CdrReportDetailInnerScheme(BaseModelScheme):
    ingress_call_cost = Float()
    egress_call_cost = Float()
    #calculated fields
    ingress_name = Str()
    egress_name = Str()
    ingress_client_name = Str()
    egress_client_name = Str()
    release_cause_name = Str()
    nrf_calls = Str()
    report_time = Int()
    asr = Float()
    qsr = Float()
    ingress_npr = Float()
    egress_npr = Float()

    class Meta:
        model = model.CdrReportDetail


class CdrReportGroppedResponseScheme(SuccessScheme):
    total = Int()
    count = Int()
    _from = Int(dump_to='from', load_from="from")
    data = Nested('CdrReportDetailInnerScheme', many=True)


class DidReportGroppedRequestScheme(CdrReportGroppedRequestScheme):
    class Meta:
        model = model.DidReport
        search_fields = ('start_time', 'end_time', 'method', 'step', 'group', 'fields', 'sort', 'order')


class DidReportInnerScheme(CdrReportDetailInnerScheme):
    ingress_name = Str()
    egress_name = Str()
    ingress_client_name = Str()
    egress_client_name = Str()
    class Meta:
        model = model.DidReport


class DidReportGroppedResponseScheme(SuccessScheme):
    total = Int()
    count = Int()
    _from = Int(dump_to='from', load_from="from")
    data = Nested('DidReportInnerScheme', many=True)


# ==
class HostBasedReportGroppedRequestScheme(CdrReportGroppedRequestScheme):
    class Meta:
        model = model.HostBasedReport
        search_fields = ('start_time', 'end_time', 'method', 'step', 'group', 'fields', 'sort', 'order')


class HostBasedReportInnerScheme(CdrReportDetailInnerScheme):
    class Meta:
        model = model.HostBasedReport


class HostBasedReportGroppedResponseScheme(SuccessScheme):
    total = Int()
    count = Int()
    _from = Int(dump_to='from', load_from="from")
    data = Nested('HostBasedReportInnerScheme', many=True)



