import os
from inv5.__version__ import __version__
import configparser

API_INI = 'api.ini'
config = configparser.ConfigParser(interpolation=None)
config.read(API_INI)
_p_changed = False


def conf_get(name, default, sec='DEFAULT'):
    global _p_changed
    if not sec in config:
        config.add_section(sec)
        _p_changed = True
    if not name in config[sec]:
        config[sec][name] = default
        _p_changed = True
        return default
    else:
        return config[sec][name]


VERSION = __version__
LOG_FILE = conf_get('log_file', '')
LOG_LEVEL = conf_get('log_level', 'debug')
LOG_TO_CONSOLE = True if conf_get('log_to_console', 'True') == 'True' else False
LOG_FORMAT = conf_get('log_format', '%(asctime)s [%(levelname)s] \'%(message)s\' at %(filename)s: %(lineno)s')
ALLOW_ORIGINS = 'http://localhost'
ALLOW_ORIGIN = conf_get('allow_origin', '*').split(' ')
API_TITLE = 'Invoice v5 API'
API_HOST = conf_get('api_host', 'localhost')
API_PORT = conf_get('api_port', '8000')
API_SCHEME = conf_get('api_schema', 'http://')
API_BASE_PATH = conf_get('api_base_path', '/v1')
API_TEST_ROUTES = conf_get('api_test_routes', 'False')
TEST_TO = conf_get('test_to', '')
AUTO_INVOICE_DEBUG = True if conf_get('auto_invoice_debug', 'False') in ('True', 'true', '1') else False
DB_CONN_STRING = conf_get('db_conn_string', 'postgresql://webbackend:password@127.0.0.1:5432/class4_dnl')
DB_POOLSIZE = conf_get('db_pool_size', '1')
FALCON_SETTING_MODULE = 'inv5.settings'
INVOICE_LOGO_URL = conf_get('invoice_logo_url','http://localhost/client_logo?client_id={client_id}')
CREATE_TABLES = True if conf_get('create_tables', 'False') == 'True' else False

JWT_SIGNATURE = conf_get('jwt_signature', '8<2V*cd(7gBn>sIon#XJ[NLat%[HXhzG,Xi{Fawu{dayS0ef-!RrE|\WUbm[-CE}')
JWT_TTL_DAYS = int(conf_get('jwt_ttl_days', '14'))
JWT_REFRESH_THRESHOLD_DAYS = int(conf_get('jwt_refresh_threshold_days', '7'))

DEFAULT_ITEMS_PER_PAGE = int(conf_get('default_items_per_page', '30'))

AUTH_END_POINT = '/auth'

FAST_TESTING = True if conf_get('fast_testing', 'False') == 'True' else False

# MIDDLEWARE_CLASSES = ['falcon_rest.contrib.middleware']
MIDDLEWARE = ['inv5.middleware.Middleware']
# MIDDLEWARE = ['falcon_rest.contrib.middleware.Middleware']
AUTH_MODULE = ''
# AUTH_MODULE = 'inv5.auth'
# AUTH_USER_MODEL = 'inv5.models.user.User'
#PERMISSIONS_MODULE = 'inv5.permissions'
PERMISSIONS_MODULE = ''
# TESTING_APP_CREATION_FUNC = 'inv5.create_app'

COLLECT_ENTITIES = True

PATHS_NOT_NEEDED_AUTH = (
    '{}{}'.format(API_BASE_PATH, AUTH_END_POINT),
    '{}/user/auth'.format(API_BASE_PATH),
    '{}/registration/create'.format(API_BASE_PATH),
    '{}/auth/reset_email'.format(API_BASE_PATH),
    '{}/auth/reset_password'.format(API_BASE_PATH),
    '{}/product/public'.format(API_BASE_PATH),
    '{}/swagger.yaml'.format(API_BASE_PATH),
    '{}/swagger.json'.format(API_BASE_PATH)
)

ALWAYS_ALLOWED_AUTH_IPS = conf_get('always_allowed_auth_ips', '127.0.0.1').split(' ')

FILES = {

    'pdf_path': conf_get('pdf_path', '/opt/denovo/web/download/invoice' , 'FILES'),
    'upload_to': conf_get('upload_to', '/opt/denovo/inv5/files', 'FILES'),
}
CELERY = {
    'CELERY_BROKER_URL': conf_get('celery_broker_url', 'redis://localhost:6379/0', 'CELERY'),
    'CELERY_RESULT_BACKEND': conf_get('celery_result_backend', 'redis://localhost:6379/0', 'CELERY'),
    'CELERYD_TASK_SOFT_TIME_LIMIT': conf_get('celeryd_task_soft_time_limit', '300', 'CELERY')
}

# SENTRY_URL = conf_get('sentry_url',
#                       'http://c456a964e8c948f39691d7d32917c8fb:178b74a88dee40f8b787622069769338@sentry.denovolab.com:9000/10')
# if SENTRY_URL=='False':
#     SENTRY_URL=None

import sys


def get_class_attr_from_string_notation(class_string):
    class_module_name, class_name = class_string.rsplit('.', 1)
    __import__(class_module_name)
    cls = sys.modules[class_module_name]
    return getattr(cls, class_name)


middleware = None


def get_middleware():
    global middleware
    if MIDDLEWARE:
        if not middleware:
            middleware = []
            for m in MIDDLEWARE:
                middleware.append(get_class_attr_from_string_notation(m)())
    return middleware


auth_module = None


def get_auth_module():
    global auth_module
    if not auth_module and AUTH_MODULE:
        __import__(AUTH_MODULE)
        auth_module = sys.modules[AUTH_MODULE]
    return auth_module

permissions_module = None
def get_permissions_module():
    global permissions_module
    if not permissions_module and PERMISSIONS_MODULE:
        __import__(PERMISSIONS_MODULE)
        permissions_module = sys.modules[PERMISSIONS_MODULE]
    return permissions_module


if _p_changed:
    with open(API_INI, 'w') as configfile:
        config.write(configfile)
