from inv5 import settings
from inv5.logger import log
import celery
from celery import Celery
from celery.signals import task_prerun, task_postrun
from celery.schedules import crontab
from inv5.model import BaseModel
from inv5.settings import DB_CONN_STRING

# settings.LOG_FILE = (settings.LOG_FILE).split('.')[0] + '_' + __name__ + '.log'


log.debug('task db:{}'.format(DB_CONN_STRING))
db = None
from inv5 import model

if '_databases' in globals() and 'default' in globals()['_databases']:
    db = globals()['_databases']['default']
if not db:
    db = model.init_db(DB_CONN_STRING, 'default', BaseModel)
app = Celery('task', broker=settings.CELERY['CELERY_BROKER_URL'])
app.conf.update(settings.CELERY)
app.conf.update(CELERY_ENABLE_UTC=True, )


def custom_logger(name):
    import os, logging
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    handler = logging.FileHandler(os.path.join('./', name + '.log'), 'w+')
    formatter = logging.Formatter("%(asctime)s:%(levelname)s:%(name)s:%(message)s")
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    return logger


class SqlAlchemyTask(celery.Task):
    """An abstract Celery Task that ensures that the connection the the
    database is closed on task completion"""

    def xx_after_return(self, status, retval, task_id, args, kwargs, einfo):
        try:
            db.session.commit()
        except:
            try:
                db.session.rollback()
            except:
                pass
        finally:
            db.session.remove()


@task_prerun.connect
def task_prerun(*args, **kwargs):
    db.engine.dispose()


@task_postrun.connect
def close_session(*args, **kwargs):
    # Flask SQLAlchemy will automatically create new sessions for you from
    # a scoped session factory, given that we are maintaining the same app
    # context, this ensures tasks have a fresh session (e.g. session errors
    # won't propagate across tasks)
    db.session.remove()


@app.on_after_configure.connect
def setup_periodic_tasks(sender, **kwargs):
    try:
        from inv5.tasks.invoice import do_auto_invoice_task
        sender.add_periodic_task(crontab(minute=0), do_auto_invoice_task)
    except Exception as e:
        log.error('cannot start do_auto_invoice_task!{}'.format(str(e)))
    try:
        from inv5.tasks.alerts import do_trunk_pending_suspension_notice, do_daily_cdr_delivery, \
            do_daily_balance_summary,do_daily_usage_summary, do_notify_zero_balance, do_notify_client_balance, \
            do_clear_last_lowbalance_send_time, do_frund_detection,do_monitored_rule_history
        sender.add_periodic_task(crontab(minute='*/10'), do_clear_last_lowbalance_send_time)
        sender.add_periodic_task(crontab(minute=0), do_notify_client_balance)
        sender.add_periodic_task(crontab(minute='*/5'), do_notify_zero_balance)
        sender.add_periodic_task(crontab(minute=30), do_daily_usage_summary)
        sender.add_periodic_task(crontab(minute=35), do_daily_balance_summary)
        sender.add_periodic_task(crontab(minute=40), do_daily_cdr_delivery)
        sender.add_periodic_task(crontab(minute=0), do_trunk_pending_suspension_notice)
        sender.add_periodic_task(crontab(minute='*/5'), do_frund_detection)
        sender.add_periodic_task(crontab(minute='*/5'), do_monitored_rule_history)
    except Exception as e:
        log.error('cannot start do_alert_task!{}'.format(str(e)))


if __name__ == '__main__':
    pass
